@@ -5,7 +5,7 @@
  *
  * Copyright (C) 2005-2006  NTT DATA CORPORATION
  *
- * Version: 1.0.2 2006/02/07
+ * Version: 1.1.1   2006/05/15
  *
  */
 #include <stdio.h>
@@ -223,10 +223,6 @@ static int AddFileACL(const char *filename, WORD perm, const int index) {
 		printf("AddFileACL: ERROR: domain is out of range.\n");
 		return -EINVAL;
 	}
-	if (!IsCorrectPath(filename, 1)) {
-		fprintf(stderr, "AddFileACL: Invalid pathname '%s'\n", filename);
-		return -EINVAL;
-	}
 	if (perm > 7) { /* Should I allow perm == 0, for somebody may wish to give '4 /etc/\*' and '0 /etc/shadow' ? */
 		fprintf(stderr, "AddFileACL: Invalid permission '%d %s'\n", perm, filename);
 		return -EINVAL;
