@@ -5,11 +5,13 @@
  *
  * Copyright (C) 2005-2006  NTT DATA CORPORATION
  *
- * Version: 1.0 2005/11/11
+ * Version: 1.1   2006/04/01
  *
  * This program checks the existence for pathnames which have the write permission,
  * and displays the pathname if nonexistent, for such pathnames are likely temporary.
  *
+ * This program can also check the existence for all pathnames,
+ * and displays the pathname if nonexistent.
  */
 #include <stdio.h>
 #include <string.h>
@@ -18,19 +20,6 @@
 #include <sys/stat.h>
 #include <stdlib.h>
 
-static void NormalizeLine(unsigned char *buffer) {
-	unsigned char *sp = buffer, *dp = buffer;
-	int first = 1;
-	while (*sp && (*sp <= 32 || 127 <= *sp)) sp++;
-	while (*sp) {
-		if (!first) *dp++ = ' ';
-		first = 0;
-		while (32 < *sp && *sp < 127) *dp++ = *sp++;
-		while (*sp && (*sp <= 32 || 127 <= *sp)) sp++;
-	}
-	*dp = '\0';
-}
-
 static int decode(const char *ascii, char *bin) {
 	char c, d, e;
 	while ((c = *bin++ = *ascii++) != '\0') {
@@ -66,18 +55,49 @@ int main(int argc, char *argv[]) {
 	const char **pattern_list = NULL;
 	int pattern_list_count = 0;
 	int i, j;
-	static char buffer[16384], buffer2[sizeof(buffer)];
-	while (memset(buffer, 0, sizeof(buffer)), fgets(buffer, sizeof(buffer) - 1, stdin) != NULL) {
-		struct stat buf;
-		char *cp = strchr(buffer, '\n');
-		int perm;
-		if (cp) *cp = '\0';
-		else if (!feof(stdin)) break;
-		if (sscanf(buffer, "%u", &perm) != 1 || (perm & 2) == 0 || (cp = strchr(buffer, ' ')) == NULL) continue;
-		memmove(buffer, cp, strlen(cp) + 1);
-		NormalizeLine(buffer);
-		if (buffer[0] != '/' || !decode(buffer, buffer2)) continue;
-		if (lstat(buffer2, &buf) == 0) continue;
+	int detect_all = 0;
+	static char buffer[16384];
+	if (argc > 1) {
+		if (strcmp(argv[1], "--all") == 0) {
+			detect_all = 1;
+		} else {
+			printf("%s [--all] < domain_policy\n\n", argv[0]);
+			printf("If --all is given, all nonexistent pathnames are shown.\n");
+			printf("If --all is not given, only nonexistent pathnames that have write permission are shown.\n");
+			return 0;
+		}
+	}
+	while (memset(buffer, 0, sizeof(buffer)), fscanf(stdin, "%16380s", buffer) == 1) {
+		static int skip_next = 0;
+		if (detect_all == 0) {
+			unsigned int perm;
+			/*
+			 * Strictly speaking, I have to ignore the following 2 cases.
+			 *
+			 *   Domain definition:  <domain> /sbin/init /some_dir/some_program
+			 *   Signal definition:  allow_signal 2 <domain> /sbin/init /some_dir/some_program
+			 *
+			 * But if /some_dir/some_program doesn't exist,
+			 * the domain defined will never be used or the signal ACLs given will never be used.
+			 * So I will display such pathnames anyway, even though the user wants to display
+			 * only nonexistent pathnames that have write permission.
+			 */
+			if (sscanf(buffer, "%u", &perm) == 1 && (perm & 2) == 0) {
+				skip_next = 1;
+				continue;
+			}
+		}
+		if (skip_next) {
+			skip_next = 0;
+			continue;
+		}
+		if (buffer[0] != '/') continue;
+		{
+			static char buffer2[sizeof(buffer)];
+			struct stat buf;
+			if (!decode(buffer, buffer2)) continue;
+			if (lstat(buffer2, &buf) == 0) continue;
+		}
 		for (i = 0; i < pattern_list_count; i++) {
 			if (strcmp(pattern_list[i], buffer) == 0) break;
 		}
