@@ -5,7 +5,7 @@
  *
  * Copyright (C) 2005-2006  NTT DATA CORPORATION
  *
- * Version: 1.1.1   2006/05/15
+ * Version: 1.1.2   2006/06/02
  *
  * This program loads policies from /root/security/ directory.
  * You can call this program to reload policies after editing policy files.
@@ -141,104 +141,51 @@ int main(int argc, char *argv[]) {
 
 	if (load_domain_policy) {
 		int new_index;
-		unsigned char *flag_list = NULL;
-		const int delete_domain_fd = open("/proc/ccs/policy/delete_domain", O_WRONLY);
-		const int update_domain_fd = open("/proc/ccs/policy/update_domain", O_WRONLY);
 		FILE *proc_fp = fopen("/proc/ccs/policy/domain_policy", "w");
-		if (delete_domain_fd == EOF || update_domain_fd == EOF || !proc_fp) {
-			fprintf(stderr, "Can't open /proc/ccs/policy/\n");
-			close(delete_domain_fd); close(update_domain_fd);
-			if (proc_fp) fclose(proc_fp);
+		if (!proc_fp) {
+			fprintf(stderr, "Can't open /proc/ccs/policy/domain_policy\n");
 			goto out_domain;
 		}
 		ReadDomainPolicy("domain_policy.txt", 0);
 		ReadDomainPolicy("/proc/ccs/policy/domain_policy", 1);
 		if (domain_list_count[0] == 0) {
 			fprintf(stderr, "Can't open domain_policy.txt\n");
-			close(delete_domain_fd); close(update_domain_fd); fclose(proc_fp);
+			fclose(proc_fp);
 			goto out_domain;
 		}
-		flag_list = (char *) malloc(domain_list_count[1]);
-		memset(flag_list, 0, domain_list_count[1]);
 		for (new_index = 0; new_index < domain_list_count[0]; new_index++) {
 			const char *domainname = domain_list[0][new_index].domainname;
 			const char **new_string_ptr = domain_list[0][new_index].string_ptr;
 			const int new_string_count = domain_list[0][new_index].string_count;
 			const int old_index = FindDomain(domainname, 1);
-			int i;
-			if (refresh_policy) {
-				if (old_index >= 0) {
-					/*
-					 * Old policy for this domain found.
-					 * We need to compare the old policy and new policy to determine
-					 * whether it is possible to continue using memory used by old policy.
-					 * If new policy includes all of old policy, we can continue
-					 * using memory used by old policy by just appending new policy.
-					 */
-					const char **old_string_ptr = domain_list[1][old_index].string_ptr;
-					const int old_string_count = domain_list[1][old_index].string_count;
-					if (new_string_count < old_string_count) {
-					delete_old_domain: ;
-						/* Need to delete old policy. */
-						const int len = strlen(domainname);
-						write(delete_domain_fd, domainname, len);
-						write(delete_domain_fd, "\n", 1);
-						/* Need to notify later. */
-						flag_list[old_index] = 1;
-					} else { /* new_string_count >= old_string_count */
-						int j;
-						for (j = 0; j < old_string_count; j++) {
-							int k;
-							if (new_string_ptr[j] == old_string_ptr[j]) continue;
-							for (k = 0; k < new_string_count; k++) {
-								if (new_string_ptr[k] == old_string_ptr[j]) break;
-							}
-							if (k < new_string_count) continue;
-							goto delete_old_domain;
-						}
-						/* Needn't to delete old policy. */
-						flag_list[old_index] = -1;
+			int i, j;
+			if (refresh_policy && old_index >= 0) {
+				/* Old policy for this domain found. */
+				const char **old_string_ptr = domain_list[1][old_index].string_ptr;
+				const int old_string_count = domain_list[1][old_index].string_count;
+				fprintf(proc_fp, "select %s\n", domainname);
+				for (j = 0; j < old_string_count; j++) {
+					for (i = 0; i < new_string_count; i++) {
+						if (new_string_ptr[i] == old_string_ptr[j]) break;
 					}
-				} else {
-					/*
-					 * Old policy for this domain not found.
-					 * We need to delete old policy leter,
-					 * indicated by "flag_list[old_index] == 0".
-					 */
-				}
-			}
-			/* Append new policy. */
-			fprintf(proc_fp, "%s\n\n", domainname);
-			for (i = 0; i < new_string_count; i++) {
-				fprintf(proc_fp, "%s\n", new_string_ptr[i]);
-			}
-			fprintf(proc_fp, "\n");
-			fflush(proc_fp);
-			if (refresh_policy) {
-				/* Notify policy update. */
-				if (old_index >= 0 && flag_list[old_index] == 1) {
-					const int len = strlen(domainname);
-					write(update_domain_fd, domainname, len);
-					write(update_domain_fd, "\n", 1);
+					/* Delete this entry from old policy if not found in new policy. */
+					if (i == new_string_count) fprintf(proc_fp, "delete %s\n", old_string_ptr[j]);
 				}
+			} else {
+				/* Old policy for this domain not found or Append to old policy. */
+				fprintf(proc_fp, "%s\n", domainname);
 			}
+			for (i = 0; i < new_string_count; i++) fprintf(proc_fp, "%s\n", new_string_ptr[i]);
+			if (old_index >= 0) domain_list[1][old_index].domainname = ""; /* Don't delete this domain later. */
 		}
-		fclose(proc_fp);
 		if (refresh_policy) {
 			int old_index;
 			/* Delete all domains that are not defined in new policy. */
 			for (old_index = 0; old_index < domain_list_count[1]; old_index++) {
-				if (flag_list[old_index] == 0) {
-					const char *domainname = domain_list[1][old_index].domainname;
-					const int len = strlen(domainname);
-					write(delete_domain_fd, domainname, len);
-					write(delete_domain_fd, "\n", 1);
-				}
+				if (domain_list[1][old_index].domainname[0]) fprintf(proc_fp, "delete %s\n", domain_list[1][old_index].domainname);
 			}
 		}
-		free(flag_list);
-		close(delete_domain_fd);
-		close(update_domain_fd);
+		fclose(proc_fp);
 	}
  out_domain: ;
 
