@@ -5,7 +5,7 @@
  *
  * Copyright (C) 2005-2006  NTT DATA CORPORATION
  *
- * Version: 1.1.1   2006/05/15
+ * Version: 1.1.2   2006/06/02
  *
  */
 #include <stdio.h>
@@ -17,48 +17,116 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
-#define ROOT_NAME "<kernel>"
-
-#define MAXBUFSIZE  8192
-
 static void OutOfMemory(void) {
 	fprintf(stderr, "Out of memory. Aborted.\n");
 	exit(1);
 }
 
+/* Copied from kernel source. */
+static inline unsigned long partial_name_hash(unsigned long c, unsigned long prevhash) {
+	return (prevhash + (c << 4) + (c >> 4)) * 11;
+}
+
+/* Copied from kernel source. */
+static inline unsigned int full_name_hash(const unsigned char *name, unsigned int len) {
+	unsigned long hash = 0;
+	while (len--) hash = partial_name_hash(*name++, hash);
+	return (unsigned int) hash;
+}
+
 #define PAGE_SIZE  4096
 
-static const char *SaveName(const char *name) {
+static char *alloc_element(const unsigned int size) {
 	static char *buf = NULL;
-	static int buf_used_len = PAGE_SIZE;
-	int i, len;
-	char *saved_name = NULL;
-	char **new_ptr = NULL;
-	static char **search_list = NULL;
-	static int search_list_count = 0;
+	static unsigned int buf_used_len = PAGE_SIZE;
+	char *ptr = NULL;
+	if (size > PAGE_SIZE) return NULL;
+	if (buf_used_len + size > PAGE_SIZE) {
+		if ((ptr = malloc(PAGE_SIZE)) == NULL) OutOfMemory();
+		buf = ptr;
+		memset(buf, 0, PAGE_SIZE);
+		buf_used_len = size;
+		ptr = buf;
+	} else if (size) {
+		int i;
+		ptr = buf + buf_used_len;
+		buf_used_len += size;
+		for (i = 0; i < size; i++) if (ptr[i]) OutOfMemory();
+	}
+	return ptr;
+}
+
+#define MAX_HASH 256
+
+typedef struct name_entry {
+	struct name_entry *next; /* Pointer to next record. NULL if none.             */
+	unsigned int hash;       /* hash and length                                   */
+	const char *name;        /* Text form of filename and domainname. Never NULL. */
+} NAME_ENTRY;
+
+typedef struct free_memory_block_list {
+	struct free_memory_block_list *next; /* Pointer to next record. NULL if none. */
+	char *ptr;                           /* Pointer to a free area.               */
+	int len;                             /* Length of the area.                   */
+} FREE_MEMORY_BLOCK_LIST;
+
+static const char *SaveName(const char *name) {
+	static FREE_MEMORY_BLOCK_LIST fmb_list = { NULL, NULL, 0 };
+	static NAME_ENTRY name_list[MAX_HASH]; /* The list of names. */
+	NAME_ENTRY *ptr, *prev = NULL;
+	unsigned int hash;
+	FREE_MEMORY_BLOCK_LIST *fmb = &fmb_list;
+	int len;
+	static int first_call = 1;
 	if (!name) return NULL;
 	len = strlen(name) + 1;
 	if (len > PAGE_SIZE) {
-		printf("ERROR: Name too long for SaveName().\n");
+		fprintf(stderr, "ERROR: Name too long for SaveName().\n");
 		return NULL;
 	}
-	for (i = 0; i < search_list_count; i++) {
-		if (strcmp(name, search_list[i]) == 0) return search_list[i];
+	hash = full_name_hash((const unsigned char *) name, len - 1);
+	if (first_call) {
+		int i;
+		first_call = 0;
+		memset(&name_list, 0, sizeof(name_list));
+		for (i = 0; i < MAX_HASH; i++) name_list[i].name = "/";
 	}
-	if (buf_used_len + len > PAGE_SIZE) {
-		if ((buf = malloc(PAGE_SIZE)) == NULL) OutOfMemory();
-		memset(buf, 0, PAGE_SIZE);
-		buf_used_len = 0;
+	ptr = &name_list[hash % MAX_HASH];
+	hash ^= len; /* The hash % MAX_HASH are always same for ptr->hash, so embed length into the hash value. */
+	while (ptr) {
+		if (hash == ptr->hash && strcmp(name, ptr->name) == 0) goto out;
+		prev = ptr; ptr = ptr->next;
 	}
-	saved_name = buf + buf_used_len;
-	memmove(saved_name, name, len);
-	if ((new_ptr = (char **) realloc(search_list, (search_list_count + 1) * sizeof(char *))) == NULL) OutOfMemory();
-	search_list = new_ptr;
-	search_list[search_list_count++] = saved_name;
-	buf_used_len += len;
-	return (const char *) saved_name;
+	while (len > fmb->len) {
+		if (fmb->next) {
+			fmb = fmb->next;
+		} else {
+			char *cp;
+			if ((cp = (char *) malloc(PAGE_SIZE)) == NULL || (fmb->next = (FREE_MEMORY_BLOCK_LIST *) alloc_element(sizeof(FREE_MEMORY_BLOCK_LIST))) == NULL) OutOfMemory();
+			memset(cp, 0, PAGE_SIZE);
+			fmb = fmb->next;
+			fmb->ptr = cp;
+			fmb->len = PAGE_SIZE;
+		}
+	}
+	if ((ptr = (NAME_ENTRY *) alloc_element(sizeof(NAME_ENTRY))) == NULL) OutOfMemory();
+	memset(ptr, 0, sizeof(NAME_ENTRY));
+	ptr->hash = hash;
+	ptr->name = fmb->ptr;
+	memmove(fmb->ptr, name, len);
+	fmb->ptr += len;
+	fmb->len -= len;
+	prev->next = ptr; /* prev != NULL because name_list is not empty. */
+	if (fmb->len == 0) {
+		FREE_MEMORY_BLOCK_LIST *ptr = &fmb_list;
+		while (ptr->next != fmb) ptr = ptr->next; ptr->next = fmb->next;
+	}
+ out:
+	return ptr ? (const char *) ptr->name : NULL;
 }
 
+#define ROOT_NAME "<kernel>"
+
 static int IsCorrectDomain(const unsigned char *domainname) {
 	unsigned char c, d, e;
 	if (!domainname || strncmp(domainname, ROOT_NAME, strlen(ROOT_NAME))) goto out;
@@ -111,19 +179,21 @@ static int AddStringEntry(const char *entry, const int index, const int type) {
 	const char *cp;
 	int i;
 	if (index < 0 || index >= domain_list_count[type]) {
-		printf("AddStringEntry: ERROR: domain is out of range.\n");
+		fprintf(stderr, "AddStringEntry: ERROR: domain is out of range.\n");
 		return -EINVAL;
 	}
 	if (!entry || !*entry) return -EINVAL;
+	if ((cp = SaveName(entry)) == NULL) OutOfMemory();
+
 	acl_ptr = domain_list[type][index].string_ptr;
 	acl_count = domain_list[type][index].string_count;
 
 	// Check for the same entry.
 	for (i = 0; i < acl_count; i++) {
-		if (strcmp(entry, acl_ptr[i]) == 0) return 0;
+		// Faster comparison, for they are SaveName'd.
+		if (cp == acl_ptr[i]) return 0;
 	}
 
-	if ((cp = SaveName(entry)) == NULL) OutOfMemory();
 	if ((acl_ptr = (const char **) realloc(acl_ptr, (acl_count + 1) * sizeof(const char *))) == NULL) OutOfMemory();
 	acl_ptr[acl_count++] = cp;
 	domain_list[type][index].string_ptr = acl_ptr;
@@ -200,28 +270,49 @@ static void SortPolicy(const int type) {
 	}
 }
 
+#define MAXBUFSIZE  8192
+
 static char buffer[MAXBUFSIZE];
 
 static void ReadDomainPolicy(const char *filename, const int type) {
-	FILE *fp;
+	FILE *fp = stdin;
 	int index;
-	if ((fp = fopen(filename, "r")) == NULL) {
-		fprintf(stderr, "Can't open %s\n", filename);
-		return;
+	if (filename) {
+		if ((fp = fopen(filename, "r")) == NULL) {
+			fprintf(stderr, "Can't open %s\n", filename);
+			return;
+		}
 	}
 	index = EOF;
 	while (memset(buffer, 0, sizeof(buffer)), fgets(buffer, sizeof(buffer) - 1, fp) != NULL) {
 		char *cp = strchr(buffer, '\n');
-		int perm;
 		if (cp) *cp = '\0';
 		else if (!feof(fp)) break;
 		NormalizeLine(buffer);
 		if (IsDomainDef(buffer)) {
 			index = FindOrAssignNewDomain(buffer, type);
-		} else if (index >= 0) {
-			if (sscanf(buffer, "%d", &perm) == 1 || strncmp(buffer, "allow_", 6) == 0) AddStringEntry(buffer, index, type);
+		} else if (index >= 0 && buffer[0]) {
+			AddStringEntry(buffer, index, type);
 		}
 	}
-	fclose(fp);
+	if (fp != stdin) fclose(fp);
 	SortPolicy(type);
 }
+
+static int WriteDomainPolicy(const int fd, const int type) {
+	int i, j;
+	for (i = 0; i < domain_list_count[type]; i++) {
+		const char **string_ptr = domain_list[type][i].string_ptr;
+		const int string_count = domain_list[type][i].string_count;
+		int len = strlen(domain_list[0][i].domainname);
+		write(fd, domain_list[type][i].domainname, len);
+		write(fd, "\n\n", 2);
+		for (j = 0; j < string_count; j++) {
+			len = strlen(string_ptr[j]);
+			write(fd, string_ptr[j], len);
+			write(fd, "\n", 1);
+		}
+		write(fd, "\n", 1);
+	}
+	return 0;
+}
