@@ -5,7 +5,7 @@
  *
  * Copyright (C) 2005-2006  NTT DATA CORPORATION
  *
- * Version: 1.1   2006/04/01
+ * Version: 1.1.2   2006/06/02
  *
  */
 #include "include.h"
@@ -375,7 +375,29 @@ int main(int argc, char *argv[]) {
 			listen(ipv4_listener_socket, 512);
 			ipv4_pid = fork();
 			if (ipv4_pid == 0) {
-				while (1) accept(ipv4_listener_socket, (struct sockaddr *) &addr, &size);
+				fd_set fds;
+				FD_ZERO(&fds);
+				FD_SET(ipv4_listener_socket, &fds);
+				while (1) {
+					fd_set temp_fds = fds;
+					int fd, fd2;
+					if (select(1024, &temp_fds, NULL, NULL, NULL) == EOF) {
+						fprintf(stderr, "select=%s\n", strerror(errno));
+						continue;
+					}
+					for (fd = 0; fd < 1024; fd++) {
+						if (!FD_ISSET(fd, &temp_fds)) continue;
+						if (fd == ipv4_listener_socket) {
+							size = sizeof(addr);
+							fd2 = accept(ipv4_listener_socket, (struct sockaddr *) &addr, &size);
+							if (fd2 != EOF) FD_SET(fd2, &fds);
+							else fprintf(stderr, "accept=%s\n", strerror(errno));
+						} else {
+							//fprintf(stderr, "closed %d\n", fd);
+							close(fd); FD_CLR(fd, &fds);
+						}
+					}
+				}
 			}
 		}
 		{
@@ -392,7 +414,29 @@ int main(int argc, char *argv[]) {
 			listen(ipv6_listener_socket, 512);
 			ipv6_pid = fork();
 			if (ipv6_pid == 0) {
-				while (1) accept(ipv6_listener_socket, (struct sockaddr *) &addr, &size);
+				fd_set fds;
+				FD_ZERO(&fds);
+				FD_SET(ipv6_listener_socket, &fds);
+				while (1) {
+					fd_set temp_fds = fds;
+					int fd, fd2;
+					if (select(1024, &temp_fds, NULL, NULL, NULL) == EOF) {
+						fprintf(stderr, "select=%s\n", strerror(errno));
+						continue;
+					}
+					for (fd = 0; fd < 1024; fd++) {
+						if (!FD_ISSET(fd, &temp_fds)) continue;
+						if (fd == ipv6_listener_socket) {
+							size = sizeof(addr);
+							fd2 = accept(ipv6_listener_socket, (struct sockaddr *) &addr, &size);
+							if (fd2 != EOF) FD_SET(fd2, &fds);
+							else fprintf(stderr, "accept=%s\n", strerror(errno));
+						} else {
+							//fprintf(stderr, "closed %d\n", fd);
+							close(fd); FD_CLR(fd, &fds);
+						}
+					}
+				}
 			}
 		}
 
