@@ -5,12 +5,12 @@
  *
  * Copyright (C) 2005-2006  NTT DATA CORPORATION
  *
- * Version: 1.1   2006/04/01
+ * Version: 1.1.2   2006/06/02
  *
  */
 #include "include.h"
 
-#define MAX_FILES 6
+#define MAX_FILES 7
 
 int main(int argc, char *argv[]) {
 	int fd, len, i;
@@ -21,6 +21,7 @@ int main(int argc, char *argv[]) {
 		"/proc/ccs/info/trusted_pids",
 		"/proc/ccs/info/deleted_pids",
 		"/proc/ccs/info/meminfo",
+		"/proc/ccs/info/mapping",
 		"/proc/ccs/info/self_domain"
 	};
 	memset(buffer, 0, sizeof(buffer));
